'''
ES 查询组件对外提供的接口
希望能够通过配置实现以下格式的调用:
    door = ESDoor(some_doc_type, condition_1=value_1, condition_2=value_2...)
    res = door.main()  # 即可查询结果
'''
from sparrow_order_lib.es.constants import IndexMapping
from sparrow_order_lib.es.es_util.constants import ES_QUERY_PAGE_SIZE_DEFAULT
from sparrow_order_lib.es.es_util.in_query_param import InQueryParam
from sparrow_order_lib.es.es_util.es_query_util import ESQueryUtil
from sparrow_order_lib.es.es_util.es_builder import ESBuilder

from sparrow_order_lib.core.datastructures import ImmutableDict


class ESDoor(object):
    '''  '''

    def __init__(self, query_type: str, **kwargs):
        self.doc_type = query_type
        self.__index = IndexMapping[self.doc_type]
        self.query_kwargs = kwargs
        try:
            self.page = kwargs.pop('page')
            self.page = int(self.page) if self.page else 0
        except:
            self.page = 1
        try:
            self.page_size = kwargs.pop('page_size')
            self.page_size = int(self.page_size) if self.page_size else ES_QUERY_PAGE_SIZE_DEFAULT
        except:
            self.page_size = ES_QUERY_PAGE_SIZE_DEFAULT
        try:
            self.sort_string = kwargs.pop('sort')
        except KeyError:
            self.sort_string = ''

        self.__kwargs = ImmutableDict(kwargs)
        self.__query_params = None
        self.__es_params = None
        self.__es_query_util = None

        self.get_cached_dsl()
        self.format_in_params()
        self.format_es_params()

    def get_cached_dsl(self):
        # 获取缓存的 dsl
        pass

    @property
    def es_query_util(self) -> ESQueryUtil:
        if self.__es_query_util is None:
            self.__es_query_util = ESQueryUtil(self.doc_type)
        return self.__es_query_util

    @property
    def es_params(self):
        return self.__es_params

    def main(self):
        ''' 对外提供统一接口, 负责查询数据 '''
        builder = ESBuilder(index=self.__index)
        for query_group in self.es_params:
            builder.addQueryGroup(query_group)
        builder.addQueryGroup(self.es_query_util.get_page_param_group(self.page, self.page_size))
        if self.sort_string:
            sort_group = self.es_query_util.get_sort_param_group(self.sort_string)
            builder.addQueryGroup(sort_group)

        result = builder.executeQuery()
        items = result['hits']['hits']

        datas = [item['_source'] for item in items]

        count = builder.executeCountQuery()

        return {'count': count, 'data': datas}

    def format_in_params(self):
        self.__query_params = []
        for in_param_key, in_param_value in self.__kwargs.items():
            inpm = InQueryParam(in_param_key, in_param_value)
            self.__query_params.append(inpm)

    def format_es_params(self):
        self.__es_params = []
        for inpm_obj in self.__query_params:
            espm = self.es_query_util.get_es_query_param(inpm_obj)
            self.__es_params.append(espm)



# -*- coding: utf-8 -*-
from django.db import models
from django.core import validators


class BaseModelTimeAndDeleted(models.Model):
    '''基础 model'''
    created_time = models.DateTimeField("Date Created", blank=True, null=True, auto_now_add=True)
    updated_time = models.DateTimeField("Date Updated", blank=True, null=True, auto_now=True)
    deleted = models.BooleanField("已删除", default=False)

    class Meta:
        abstract = True


class BaseModelNew(models.Model):
    '''基础 model'''
    deleted = models.BooleanField("已删除", default=False)

    class Meta:
        abstract = True


class AbstractAddress(models.Model):

    PHONE_NUMBER_REGEX = r"1[0-9]{10}"

    title = models.CharField(
        '地址名称',
        max_length=64,
        # choices=TITLE_CHOICES,
        blank=True
    )
    name = models.CharField(
        '姓名',
        max_length=64,
        blank=True
    )
    phone = models.CharField(
        '收件人手机号',
        max_length=11,
        unique=False,
        help_text='请输入手机号',
        validators=[
            validators.RegexValidator(
                PHONE_NUMBER_REGEX,
                '手机号输入有误，请重新输入',
            ),
        ],
    )
    province = models.CharField(
        '省',
        max_length=64,
        blank=True
    )
    city = models.CharField(
        '城市',
        max_length=255,
        # choices=TITLE_CHOICES,
        blank=True
    )
    district = models.CharField(
        '区县',
        max_length=255,
        # choices=TITLE_CHOICES,
        blank=True
    )
    detail = models.CharField(
        '地址详情',
        max_length=255,
        # choices=TITLE_CHOICES,
        blank=True
    )

    def __str__(self):
        return '{0}-{1}-{2}{3}{4}{5}'.format(self.name, self.phone, self.province, self.city, self.district, self.detail)

    class Meta:
        abstract = True
        verbose_name = 'Address'
        verbose_name_plural = 'Addresses'




BASE_SHIPPING_API_CODE = 20104000


class ShippingPartnerCodes(object):
    '''ExpressPartner里面要用到的code'''
    # 顺丰
    SF = "sf"
    # 顺丰同城急送
    SFTCJS = "sftcjs"
    # 圆通
    YTO = "yto"
    # 中通
    ZTO = "zto"
    # 申通
    STO = "sto"
    # EMS
    EMS = "ems"
    # 邮政
    YZPY = "yzpy"
    # 韵达
    YD = "yd"
    # 宅急送
    ZJS = "zjs"
    # 京东
    JD = "jd"
    # 德邦
    DBL = "dbl"
    # 百世快递
    HTKY = "htky"
    # 天天
    HHTT = "hhtt"
    # 其它
    OTHERS = "others"
    # 专柜代发
    SHOP = "shop"
    # 闪送
    SS = "ss"


# 快递公司代号与中文对照
ShippingPartnerCodes_DICT = {
    ShippingPartnerCodes.SF: "顺丰",
    ShippingPartnerCodes.SFTCJS: "顺丰同城急送",
    ShippingPartnerCodes.YTO: "圆通",
    ShippingPartnerCodes.ZTO: "中通",
    ShippingPartnerCodes.STO: "申通",
    ShippingPartnerCodes.EMS: "EMS",
    ShippingPartnerCodes.YZPY: "邮政",
    ShippingPartnerCodes.YD: "韵达",
    ShippingPartnerCodes.ZJS: "宅急送",
    ShippingPartnerCodes.JD: "京东",
    ShippingPartnerCodes.DBL: "德邦",
    ShippingPartnerCodes.HTKY: "百世快递",
    ShippingPartnerCodes.HHTT: "天天",
    ShippingPartnerCodes.OTHERS: "其它",
    ShippingPartnerCodes.SHOP: "专柜代发",
    ShippingPartnerCodes.SS: "闪送",
}


class PrintStatus(object):

    NEW = "new"
    PRINTING = "printing"
    PRINTED = "printed"


class SFShippingType(object):

    SF_NEXT_DAY = "sf_next_day"
    SF_THIRD_DAY = "sf_third_day"


class ZTOExpressImportFields(object):
    '''导入中通excel中用的column name'''

    SHIPPING_NUMBER = "shipping_number"
    NOTE = "note"
    POSTAGE = "postage"

    fields = {
        # SHIPPING_NUMBER: '单号',
        SHIPPING_NUMBER: '运单编号',
        POSTAGE: '运费',
        # NOTE: '备注'
        NOTE: '商品信息'
    }


class SFExpressImportFields(object):
    '''导入顺丰excel中用的column name'''

    SHIPPING_NUMBER = "shipping_number"
    NOTE = "note"
    ASSIGNMENT_ID = "assignment_id"
    POSTAGE = "postage"

    fields = {
        SHIPPING_NUMBER: '运单号',
        ASSIGNMENT_ID: '订单号',
        NOTE: '操作备注',
        POSTAGE: "参考运费/元",
    }


# 邮费映射表
EXPRESS_PRICE_MAP = {
    "zto": {
        "北京": 6,
        "天津": 7,
        "河北": 7,
        "甘肃": 15,
        "宁夏": 15,
        "青海": 15,
        "海南": 20,
        "新疆": 20,
        "西藏": 20,
        "default": 8
    }
}


class express_pay_method(object):
    # 寄付
    SENDER_PAY = "sender_pay"
    # 到付
    RECEIVER_PAY = "receiver_pay"

    EXPRESS_PAY_METHOD_CHOISE = (
        (SENDER_PAY, "寄付"),
        (RECEIVER_PAY, "到付"),
    )
